using UnityEngine;
using System.Collections;
using System.Collections.Generic;

// -----------------------------------------------------------------------
public class BaseMapFactory : MonoBehaviour
// -----------------------------------------------------------------------
{
	static int   MAP_SIZE		    = 128;
	static float HEX_SIDELENGTH     = 1.0f;
	static float SQRT3 			    = Mathf.Sqrt(3.0f);
	static float HEX_DIST_FACTOR_X  = SQRT3 * HEX_SIDELENGTH;
	static float HEX_DIST_FACTOR_Z  = 1.5f * HEX_SIDELENGTH;
	
	static List<Vector3> m_vVerts = new List<Vector3>();
	static List<int>     m_iTris  = new List<int>();
	static List<Vector2> m_vUVs   = new List<Vector2>();
	static List<Vector3> m_vNorms = new List<Vector3>();
	static List<Color>   m_cVCol  = new List<Color>();
	
	// -----------------------------------------------------------------------
	public static void CreateMapObject(string sName, Vector3 vPos)
	// -----------------------------------------------------------------------
	{		
		CreateTileData();
		CreateObject(sName, vPos);
	}
	
	// -----------------------------------------------------------------------
	static void CreateTileData()
	// -----------------------------------------------------------------------
	{		
		for (int z=0;z<MAP_SIZE;z++)
		for (int x=0;x<MAP_SIZE;x++)
		{
			Vector3 vP = CalculateTileCenterPosition(x, z);
			m_iTris.Add(m_vVerts.Count);
			m_iTris.Add(m_vVerts.Count);
			m_iTris.Add(m_vVerts.Count);
			m_vVerts.Add(vP);
			m_vNorms.Add(new Vector3(0.0f,1.0f,0.0f));
			m_vUVs.Add(new Vector2(0.5f,0.5f));
			m_cVCol.Add(new Color(x/(MAP_SIZE-1.0f),z/(MAP_SIZE-1.0f),0.0f,0.0f));  
		}
	}
	
	// -----------------------------------------------------------------------
	static void CreateObject(string sName, Vector3 vPos)
	// -----------------------------------------------------------------------
	{		
		GameObject mc = new GameObject(sName);		
		
		MeshRenderer mr   = mc.AddComponent<MeshRenderer>();
		MeshFilter   mf   = mc.AddComponent<MeshFilter>();		
		
		mc.transform.position = vPos;
		mr.material           = Resources.Load("HextileMaterial") as Material;

		mf.sharedMesh = new Mesh();
		mf.sharedMesh.vertices  = m_vVerts.ToArray();
		mf.sharedMesh.colors    = m_cVCol.ToArray();
		mf.sharedMesh.triangles = m_iTris.ToArray();
		mf.sharedMesh.uv        = m_vUVs.ToArray();
		mf.sharedMesh.normals   = m_vNorms.ToArray();
	}

	// -----------------------------------------------------------------------
	public static Vector3 CalculateTileCenterPosition(int iX, int iZ)
	// -----------------------------------------------------------------------
	{	
		float fPosX = iX*HEX_DIST_FACTOR_X;
		float fPosZ = iZ*HEX_DIST_FACTOR_Z;
		
		if (Mathf.Abs(iZ%2)==1) fPosX += (0.5f*Mathf.Sqrt(3)*HEX_SIDELENGTH);
	
		return(new Vector3(fPosX,0.0f,fPosZ));
	}
}
